@@ -1,5 +1,6 @@
 import re
 import os
+import mimetypes
 from typing import Any, Dict, List, Union, Callable
 from docassemble.base.util import log, word, DADict, DAList, DAObject, DAFile, DAFileCollection, DAFileList, defined, value, pdf_concatenate, zip_file, DAOrderedDict, action_button_html, include_docx_template, user_logged_in, user_info, send_email, docx_concatenate, get_config, space_to_underscore, DAStaticFile, alpha
 
@@ -15,6 +16,8 @@ __all__ = ['ALAddendumField',
            'ALExhibitList',
            'ALExhibit',
            'ALExhibitDocument',
+           'ALTableDocument',
+           'ALUntransformedDocument',
            'unpack_dafilelist']
 
 DEBUG_MODE = get_config('debug')
@@ -762,8 +765,10 @@ class ALDocumentBundle(DAList):
 
   def init(self, *pargs, **kwargs):
     super().init(*pargs, **kwargs)
-    self.auto_gather=False
-    self.gathered=True
+    if not hasattr(self, 'auto_gather'):
+      self.auto_gather=False
+    if not hasattr(self, 'gathered'):
+      self.gathered=True
     self.initializeAttribute('cache', DALazyAttribute)
     self.always_enabled = hasattr(self, 'enabled') and self.enabled
 
@@ -896,11 +901,16 @@ class ALDocumentBundle(DAList):
 
   def download_list_html(self, key:str='final', format:str='pdf', view:bool=True,
       refresh:bool=True, include_zip:bool = True, view_label="View", view_icon:str="eye",
-      download_label:str="Download", download_icon:str="download", zip_label:str="Download zip",
+      download_label:str="Download", download_icon:str="download", zip_label:str=None,
       zip_icon:str="file-archive") -> str:
     """
     Returns string of a table to display a list
     of pdfs with 'view' and 'download' buttons.
+    
+    `format` is one of:
+    * pdf
+    * docx
+    * original
     """
     # Trigger some variables up top to avoid idempotency issues
     enabled_docs = self.enabled_documents(refresh=refresh)
@@ -913,18 +923,31 @@ class ALDocumentBundle(DAList):
 
     for doc in enabled_docs:
       filename_root = os.path.splitext(str(doc.filename))[0]
+      # Do our best to use the provided filename + the extension from requested filetype
+      if format=='original':
+        download_doc = doc[key]
+        download_filename = doc.filename
       if format=='docx' and doc._is_docx(key=key):
         download_doc = doc.as_docx(key=key)
         download_filename = filename_root + ".docx"
       else:
         download_doc = doc.as_pdf(key=key)
         download_filename = filename_root + ".pdf"
-
+      
+      try:
+        # If it's possible, set the file extension to the actual filetype
+        # This is mostly necessary if people omit the file extension in attachment block
+        # for filetype="original"
+        ext = next(iter(mimetypes.guess_all_extensions(download_doc.mimetype, strict=True)))
+        download_filename = filename_root + ext
+      except:
+        pass        
+      
       doc_download_button = action_button_html(
           download_doc.url_for(attachment=True, display_filename=download_filename),
           label=download_label, icon=download_icon, color="primary", size="md",
           classname='al_download' )
-      if view:
+      if view and doc.as_pdf().url_for().endswith(".pdf"):
         doc_view_button = action_button_html(
             doc.as_pdf(key=key).url_for(attachment=False, display_filename=filename_root + ".pdf"),
             label=view_label, icon=view_icon, color="secondary", size="md", classname='al_view' )
@@ -936,6 +959,8 @@ class ALDocumentBundle(DAList):
     # Add a zip file row if there's more than one doc
     filename_root = os.path.splitext(str(self.filename))[0]
     if len(enabled_docs) > 1 and include_zip:
+      if not zip_label:
+        zip_label = str(self.zip_label)      
       zip = self.as_zip(key=key)
       zip_button = action_button_html(
           zip.url_for(attachment=False, display_filename = filename_root + ".zip"),
@@ -1008,14 +1033,14 @@ class ALDocumentBundle(DAList):
 
     return_str = f'''
   <div class="al_send_bundle {name}" id="al_send_bundle_{name}" name="al_send_bundle_{name}">
-    <h5 id="al_doc_email_header">Get a copy of the documents in email</h5> 
+    <h5 id="al_doc_email_header">{self.get_email_copy}</h5> 
     '''
     if show_editable_checkbox:
       return_str += f'''
     <div class="form-check-container">
       <div class="form-check">
         <input class="form-check-input" type="checkbox" class="al_wants_editable" id="{al_wants_editable_input_id}">
-        <label class="al_wants_editable form-check-label" for="{al_wants_editable_input_id}">{word("Include an editable copy")}
+        <label class="al_wants_editable form-check-label" for="{al_wants_editable_input_id}">{self.include_editable_documents}
         </label>
       </div>
     </div>
@@ -1034,7 +1059,7 @@ class ALDocumentBundle(DAList):
 
   def send_email(self, to:any=None, key:str='final', editable:bool=False, template:any=None, **kwargs) -> bool:
     """
-    Send an email with the current bundle as a single flat pdf or as editable documents.
+    Send an email with the current bundle as a series of flat pdfs (one per bundle entry) or as editable documents.
     Can be used the same as https://docassemble.org/docs/functions.html#send_email with
     two optional additional params.
 
@@ -1051,7 +1076,7 @@ class ALDocumentBundle(DAList):
     if editable:
       return send_email(to=to, template=template, attachments=self.as_editable_list(key=key), **kwargs)
     else:
-      return send_email(to=to, template=template, attachments=self.as_pdf(key=key), **kwargs)
+      return send_email(to=to, template=template, attachments=self.as_pdf_list(key=key), **kwargs)
 
   # I don't think this was actually ever used
   def table_css(self):
@@ -1318,12 +1343,70 @@ class ALExhibitDocument(ALDocument):
     filename = os.path.splitext(self.filename)[0] + ".pdf"
     
     if len(self.exhibits):
-      return pdf_concatenate(self.table_of_contents, self.exhibits.as_pdf(add_cover_pages=self.include_exhibit_cover_pages), filename=filename)    
+      if self.include_table_of_contents:
+        return pdf_concatenate(self.table_of_contents, self.exhibits.as_pdf(add_cover_pages=self.include_exhibit_cover_pages), filename=filename)    
+      else:
+        return self.exhibits.as_pdf(add_cover_pages=self.include_exhibit_cover_pages, filename=filename)
     # pdf_concatenate([a.as_pdf() for a in self.exhibits], filename=self.filename)
 
   def as_docx(self, key:str="bool", refresh:bool=True) -> DAFile:
       return self.as_pdf()
 
+class ALTableDocument(ALDocument):
+  def init(self, *pargs, **kwargs):
+    super().init(*pargs, **kwargs)
+    self.has_addendum = False
+
+  def has_overflow(self):
+    """
+    Provided for signature compatibility with ALDocument.
+    """
+    return False
+
+  def __getitem__(self, key):
+    # This overrides the .get() method so that the 'final' and 'private' key always exist and
+    # point to the same file.
+    return self.as_pdf()
+  
+  def as_list(self, **kwargs) -> List[DAFile]:
+    return [self]
+
+  def as_pdf(self, **kwargs) -> DAFile:
+    """
+    Args:
+        key (str): unused, for signature compatibility with ALDocument
+    """
+    return self.table.export(self.filename + '.xlsx', title=self.filename)
+  
+  def as_docx(self, **kwargs) -> DAFile:
+    return self.as_pdf()
+    
+
+def ALUntransformedDocument(ALDocument):
+  def init(self, *pargs, **kwargs):
+    super().init(*pargs, **kwargs)
+    self.has_addendum = False
+
+  def has_overflow(self):
+    """
+    Provided for signature compatibility with ALDocument.
+    """
+    return False
+  
+  def as_list(self, **kwargs) -> List[DAFile]:
+    return [self]
+
+  def as_pdf(self, **kwargs) -> DAFile:
+    """
+    Args:
+        key (str): unused, for signature compatibility with ALDocument
+    """
+    return self[key]
+  
+  def as_docx(self, **kwargs) -> DAFile:
+    return self.as_pdf()
+  
+    
 def unpack_dafilelist(the_file:DAFileList)->DAFile:
   """Creates a plain DAFile out of the first item in a DAFileList
   Args:
